// CannyStill.cpp

#include<opencv2/core/core.hpp>
#include<opencv2/highgui/highgui.hpp>
#include<opencv2/imgproc/imgproc.hpp>

#include<iostream>
#include<conio.h>           // may have to modify this line if not using Windows
#include<fstream>

int M[1000][1000];

struct person { int minX; int minY; int maxX; int maxY; } P[20];
void draw(cv::Mat imgOriginal, int minX, int minY, int maxX, int maxY);

int main() {
	bool toText = false;

	cv::Mat imgOriginal;        // input image
	cv::Mat imgGrayscale;       // grayscale of input image
	cv::Mat imgBlurred;         // intermediate blured image
	cv::Mat imgCanny;           // Canny edge image

	int noPersons = 0;
	bool activePerson = false;
	bool foundPixel = false;

	imgOriginal = cv::imread("sample.jpg");          // open image

	if (imgOriginal.empty()) {                                  // if unable to open image
		std::cout << "error: image not found\n";     
		_getch();                                               
		return(0);                                              
	}


	cv::Mat imgHSV;
	cv::Mat imgThreshLow;
	cv::Mat imgThreshHigh;
	cv::Mat imgThresh;
	std::vector<cv::Vec3f> v3fCircles;
	std::vector<cv::Vec3f> v3fSquares;

	cv::cvtColor(imgOriginal, imgHSV, CV_BGR2HSV);

	cv::inRange(imgHSV, cv::Scalar(0, 155, 155), cv::Scalar(50, 255, 255), imgThreshLow);
	cv::inRange(imgHSV, cv::Scalar(165, 155, 155), cv::Scalar(179, 255, 255), imgThreshHigh);

	cv::add(imgThreshLow, imgThreshHigh, imgThresh);

	cv::GaussianBlur(imgThresh, imgThresh, cv::Size(3, 3), 0);

	cv::Mat structuringElement = cv::getStructuringElement(cv::MORPH_RECT, cv::Size(3, 3));

	cv::dilate(imgThresh, imgThresh, structuringElement);
	cv::erode(imgThresh, imgThresh, structuringElement);


	//covnert image to matrix of pixels
	for (unsigned int i = 0; i < imgThresh.rows; i++) {
		for (unsigned int j = 0; j < imgThresh.cols; j++) {
			M[i][j] = imgThresh.at<uchar>(cv::Point(j, i));
		}
	}

	//save to text file
	if (toText) {
		std::ofstream fout("data.txt");
		for (unsigned int i = 0; i < imgThresh.rows; i++) {
			for (unsigned j = 0; j < imgThresh.cols; j++) {
				int val = M[i][j];
				if (val < 10) fout << " " << val << " ";
				else if (val < 100) fout << " " << val;
				else if (val < 1000) fout << val;
				fout << " ";
			}
			fout << '\n';
		}
	}


	//determine each person
	//update min, max
	for (int i = 0; i < 20; i++) {
		P[i].maxX = 0;
		P[i].maxY = 0;
		P[i].minX = 100000;
		P[i].minY = 100000;
	}


	//distinguish persons
	for (int j = 0; j < imgThresh.cols; j++) {
		//moving to the right(in image)
		bool foundPixel = false;

		for (int i = 0; i < imgThresh.rows; i++) {
			//moving down(in image)
			if (M[i][j] > 0) {//found a non black pixel
				if (foundPixel == false) {
					foundPixel = true;
					activePerson = true;
				}
				if (i < P[noPersons].minY) { P[noPersons].minY = i; }
				if (i > P[noPersons].maxY) { P[noPersons].maxY = i; }
				if (j < P[noPersons].minX) { P[noPersons].minX = j; }
				if (j > P[noPersons].maxX) { P[noPersons].maxX = j; }
			}
		}
		if (foundPixel == false && activePerson == true) {
			noPersons++;
			activePerson = false;
		}
	}


	std::cout <<"Number of persons: " << noPersons << '\n';
	std::cout << "Their coordinates\n";
	for (int i = 0; i < noPersons; i++) {
		std::cout << "X : " << P[i].minX << " " << P[i].maxX << '\n';
		std::cout << "Y : " << P[i].minY << " " << P[i].maxY << '\n';
	}

	std::cout << "Image size:(rows-cols) " << imgOriginal.rows << " " << imgOriginal.cols << '\n';

	for (int i = 0; i < noPersons; i++) {

		//left
		draw(imgOriginal, P[i].minX, P[i].minY, P[i].minX, P[i].maxY);

		//right
		draw(imgOriginal, P[i].maxX, P[i].maxY, P[i].maxX, P[i].minY);

		//top
		draw(imgOriginal, P[i].minX, P[i].minY, P[i].maxX, P[i].minY);

		//bottom
		draw(imgOriginal, P[i].minX, P[i].maxY, P[i].maxX, P[i].maxY);
		
	}

	// declare windows
	cv::namedWindow("imgOriginal", cv::WINDOW_NORMAL);	// note: you can use CV_WINDOW_NORMAL which allows resizing the window
	cv::namedWindow("imgThresh", cv::WINDOW_NORMAL);	// or CV_WINDOW_AUTOSIZE for a fixed size window matching the resolution of the image
	
	cv::resizeWindow("imgOriginal", imgOriginal.cols/2, imgOriginal.rows/2);
	cv::resizeWindow("imgThresh", imgThresh.cols / 2, imgThresh.rows / 2);

	cv::imshow("imgOriginal", imgOriginal);			// show windows
	cv::imshow("imgThresh", imgThresh);
	cv::waitKey(0);
	return(0);
}

void draw(cv::Mat img, int x1,int y1,int x2,int y2)
{
	cv::line(img,
		cv::Point(x1, y1),
		cv::Point(x2, y2),
		cv::Scalar(0, 0, 255),
		3,
		8,
		0);
}